//---------------------------------------------------------------------------
// Title      :Rahmen fr ein AVR ATmega128 C-Programm mit LCD
// Date       :10.04.2005
// Version    :1.0
// Autor      :Denis Jakel
//---------------------------------------------------------------------------
#define 	F_CPU 14745600UL
//#define     baud 9600
#define     sbi(ADDRESS,BIT) (ADDRESS |= (1<<BIT))
#define 	cbi(ADDRESS,BIT) (ADDRESS &= ~(1<<BIT))
#define 	outp(VAL,ADRESS) ADRESS=VAL
#define 	inp(VAL) VAL

#include	<avr/io.h>
#include 	<compiler.h>
#include    <sys/timer.h>
#include    <stdio.h>
#include	<stdlib.h>
#include    <avr/iom128.h>

//////////////////////////////////////////////////////////////////////////////
//	LCD-Funktionen 
//	4-BitModus an PortD Bit 4-7
//	PortE Bit 2 = RS, high=Daten, low=Kommando
//	PortE Bit 3 = E, high-Impuls fr gltige Daten
//---------------------------------------------------------------------------
//	lcd_send(..) - sendet ein Byte an LCD im 4-Bit-Modus
//	RS muss vorher richtig gesetzt sein
//	PE:	data=zu sendendes Byte
//---------------------------------------------------------------------------
void lcd_send(char data)
{
	// aktuelles RS ermitteln
	char rs=PORTE;
	rs&=4;
	// High-Teil senden
	char tmp=data;
	tmp&=0xf0;
	tmp|=rs;
	PORTD=tmp;
	// Schreibsignal Enable high-Impuls
	sbi(PORTE,3);
	cbi(PORTE,3);
	// Low-Teil senden
	tmp=data;
	tmp&=0x0f;
	tmp*=16;
	tmp|=rs;
	PORTD=tmp;
	// Schreibsignal Enable high-Impuls
	sbi(PORTE,3);
	cbi(PORTE,3);
	// verarbeiten lassen
	NutSleep(1);
}
//---------------------------------------------------------------------------
//	lcd_cmd(..) - sendet ein Kommando an LCD
//	PE:	cmd=Kommando-Byte
//---------------------------------------------------------------------------
void lcd_cmd(char cmd)
{
	cbi(PORTE,2);		// RS lschen = Kommando
	lcd_send(cmd);		// senden
}

//---------------------------------------------------------------------------
//	lcd_write(..) - sendet ein Zeichen (Daten) an LCD
//	PE:	text=Zeichen
//---------------------------------------------------------------------------

void lcd_write(char text)
{
	sbi(PORTE,2);		// RS setzen = Daten
	lcd_send(text);		// senden
}

//---------------------------------------------------------------------------
//	lcd_write(..) - sendet eine Zeichenkette an LCD
//	Die Zeichenkette muss mit 0x00 abgeschlossen sein
//	PE:	pText=Zeiger auf Zeichenkette
//---------------------------------------------------------------------------
void lcd_write1(char* pText)
{
	while(pText[0]!=0)
	{
		lcd_write(pText[0]);
		pText++;
	}
}

//---------------------------------------------------------------------------
//	lcd_write(..) - sendet eine Zeichenkette an LCD
//	PE:	pText=Zeiger auf Zeichenkette
//		count=Anzahl der zu sendenden Zeichen
//---------------------------------------------------------------------------
void lcd_write2(char* pText, int count)
{
	while(count!=0)
	{
		lcd_write(pText[0]);
		pText++;
		count--;
	}
}

//---------------------------------------------------------------------------
//	lcd_home(..) - Cursor auf Position 1,1
//---------------------------------------------------------------------------
void lcd_home(void)
{
	lcd_cmd(0x02);
	NutSleep(2);	// warten in ms			
}
//---------------------------------------------------------------------------
//	lcd_clear(..) - lscht die Anzeige im LCD
//---------------------------------------------------------------------------
void lcd_clear(void)
{
	lcd_cmd(0x01);
	NutSleep(2);	// warten in ms
}
//---------------------------------------------------------------------------
//	lcd_on(..) - schaltet das LCD an
//---------------------------------------------------------------------------
void lcd_on(void)
{
	lcd_cmd(0x0C);
}
//---------------------------------------------------------------------------
//	lcd_off(..) - schaltet das LCD aus
//---------------------------------------------------------------------------
void lcd_off(void)
{
	lcd_cmd(0x08);
}
//---------------------------------------------------------------------------
//	lcd_goto(..) - setzt die Cursorposition
// 	PE:	row = Zeile 1..2
//		col = Spalte 1..16
//---------------------------------------------------------------------------
void lcd_goto(int row, int col)
{
	row--;				// Null-basierend
	row&=0x01;			// sicherheitshalber
	row*=0x40;			// Zeile nach Bit 6 bringen
	col--;				// Null-basierend
	col&=0x0f;			// sicherheitshalber
	char tmp=row|col;
	tmp|=0x80;			// Cursor setzen
	lcd_cmd(tmp);		// senden
}
//---------------------------------------------------------------------------
//	lcd_init(..) - Schaltet die Ports und Initialisiert das LCD
//---------------------------------------------------------------------------
void lcd_init(void)
{
	
	DDRD=0xf0;	// Port D Bit 4,5,6,7 = Ausgang
	PORTD=0;
	DDRE=0x0C;	// Port E Bit 2,3 = Ausgang
	PORTE=0;
	// warten bist LCD-Controller gebootet
	NutSleep(50);
	//wait_ms(50);		
	// SOFT-RESET
	PORTD = 0x30;	//0b00110000;
	sbi(PORTE,3);	// Schreibsignal Enable high-Impuls
	cbi(PORTE,3);
	NutSleep(5);
	PORTD = 0x30;	//0b00110000;
	sbi(PORTE,3);	// Schreibsignal Enable high-Impuls
	cbi(PORTE,3);
	NutSleep(1);
	PORTD = 0x30;	//0b00110000;
	sbi(PORTE,3);	// Schreibsignal Enable high-Impuls
	cbi(PORTE,3);
	NutSleep(5);
	

	// 4-BitModus einschalten
	PORTD=0x20;			
	// Schreibsignal
	sbi(PORTE,3);	// Schreibsignal Enable high-Impuls
	cbi(PORTE,3);
	NutSleep(5);
	

	// ab hier im 4-Bit-Modus
	lcd_cmd(0x28);		// Funktions-Set: 2 Zeilen, 5x7 Matrix, 4 Bit
	//lcd_off();
	lcd_cmd(0x06);		// Entry Mode
	lcd_on();
	lcd_clear();
}

void USART_Init( unsigned int baud )
{
/* Set baud rate */
UBRR0H = /*0x00;*/ (unsigned char)(baud>>8);
UBRR0L = /*0x5F;*/ (unsigned char)baud;
/* Enable receiver and transmitter */
UCSR0A=0x00;
/* USART operates at normal speed */
UCSR0B = (  (1<<RXEN0) | (1<<TXEN0));    /* Enable receiver */
UCSR0C = (  (0<<UMSEL) |  /* Configure for asynchronous operation */
        (0<<UPM1)  |  /* No parity */
        (0<<UPM0)  |
        (0<<USBS)  |  /* 1 stopbit */
        (1<<UCSZ1) |  /* 8 data bits */
        (1<<UCSZ0));

}

char getChar(void)
{
	char data=0;
	//warte bis RX-complete RXC UCSR0A / USR bei z.B.: AT090S2313
	while ( !( UCSR0A & (1<<RXC0)) );
	//empfangen
	data=UDR0;
	return data;
}

//----------------------------------------------------------------------
void putChar(char data)
{
	//warte bis UDR le er ist UCSRA / USR bei z.B.: 2313
	while (!(UCSR0A&32));
	//sende
	UDR0=data;
}
//---------------------------------------------------------------------------
// Main-Funktion
//---------------------------------------------------------------------------
int main (void)
{
	DDRB|=0x40;
	
	char string[32];
	int i;
	double x;
	
	//char offset=3;
	//char steigung=2;
	//int time_out;
	
	NutSleep(200);
	lcd_init();
	USART_Init(95);
	lcd_goto(1,1);
	lcd_write1("Waiting..");
	
	
	while(1) 
	{
	// Mainloop	
	
		for(i=0;i<=31;i++) string[i]=0;

		NutSleep(1000);
		PORTB|=0x40;
		putChar('1');
		putChar('\r');
		

		/*while( !( UCSR0A & (1<<RXC0)) );
		i=0;

		do
		{

			string[i]=UDR0;
			i++;
			//_delay_ms(1.4); // 1.1 - 1.3 Timing Problem behoben
			time_out=30;
			while(!( UCSR0A & (1<<RXC0)) )
			{
				if((time_out--) == 0) break;
				NutSleep(1);
			}
		}
		while( (UCSR0A & (1<<RXC0)) );*/

		string[i]=0;

		lcd_goto(1,1);
		lcd_write1("Messwert:");
		lcd_goto(2,1);
		lcd_write1("                "); //Zeile mit 16 Leerzeichen beschreiben, schneller als LCD clear
		//
		x=atof(string);
		//	x=offset-x*steigung;
		//_delay_ms(1000);
		sprintf(string,"%1.3f",123.456);
		lcd_goto(2,1);
		lcd_write1(string);

		NutSleep(1000);

	} // Mainloop
	return 0;
	
	
}

//---------------------------------------------------------------------------

